@@ -1,6 +1,7 @@
 from pyper import task
 import pytest
 
+class TestError(Exception): ...
 
 def f1(data):
     return data
@@ -9,7 +10,10 @@ def f2(data):
     yield data
 
 def f3(data):
-    raise RuntimeError
+    raise TestError
+
+def f4(data):
+    return [data]
 
 async def af1(data):
     return data
@@ -18,7 +22,7 @@ async def af2(data):
     yield data
 
 async def af3(data):
-    raise RuntimeError
+    raise TestError
 
 async def af4(data):
     async for row in data:
@@ -31,24 +35,29 @@ async def consumer(data):
     return total
 
 @pytest.mark.asyncio
-async def test_pipeline():
-    p = task(f1) | task(f2)
-    assert p(1).__next__() == 1
+async def test_aiterable_branched_pipeline():
+    p = task(af1) | task(f2, branch=True)
+    assert await p(1).__anext__() == 1
+
+@pytest.mark.asyncio
+async def test_iterable_branched_pipeline():
+    p = task(af1) | task(f4, branch=True)
+    assert await p(1).__anext__() == 1
 
 @pytest.mark.asyncio
 async def test_joined_pipeline():
-    p = task(af1) | task(af2) | task(af4, join=True)
+    p = task(af1) | task(af2, branch=True) | task(af4, branch=True, join=True)
     assert await p(1).__anext__() == 1
 
 @pytest.mark.asyncio
 async def test_consumer():
-    p = task(af1) | task(af2) > consumer
+    p = task(af1) | task(af2, branch=True) > consumer
     assert await p(1) == 1
 
 @pytest.mark.asyncio
-async def test_invalid_first_stage_concurrency():
+async def test_invalid_first_stage_workers():
     try:
-        p = task(af1, concurrency=2) | task(af2) > consumer
+        p = task(af1, workers=2) | task(af2, branch=True) > consumer
         await p(1)
     except Exception as e:
         assert isinstance(e, RuntimeError)
@@ -58,17 +67,7 @@ async def test_invalid_first_stage_concurrency():
 @pytest.mark.asyncio
 async def test_invalid_first_stage_join():
     try:
-        p = task(af1, join=True) | task(af2) > consumer
-        await p(1)
-    except Exception as e:
-        assert isinstance(e, RuntimeError)
-    else:
-        raise AssertionError
-
-@pytest.mark.asyncio
-async def test_error_handling():
-    try:
-        p = task(af1) | task(af2) | task(af3) > consumer
+        p = task(af1, join=True) | task(af2, branch=True) > consumer
         await p(1)
     except Exception as e:
         assert isinstance(e, RuntimeError)
@@ -76,17 +75,40 @@ async def test_error_handling():
         raise AssertionError
     
 @pytest.mark.asyncio
-async def test_unified_pipeline():
-    p = task(af1) | task(f1) | task(af2) | task(f2) > consumer
-    assert await p(1) == 1
-
-@pytest.mark.asyncio
-async def test_error_handling_in_daemon():
+async def test_invalid_branch_result():
     try:
-        p = task(af1) | task(af2) | task(f3, daemon=True) > consumer
+        p = task(af1, branch=True) > consumer
         await p(1)
     except Exception as e:
-        assert isinstance(e, RuntimeError)
+        assert isinstance(e, TypeError)
     else:
         raise AssertionError
-    
\ No newline at end of file
+
+async def _try_catch_error(pipeline):
+    try:
+        p = task(af1) | pipeline > consumer
+        await p(1)
+    except Exception as e:
+        return isinstance(e, TestError)
+    else:
+        return False
+    
+@pytest.mark.asyncio
+async def test_async_error_handling():
+    p = task(af3)
+    assert await _try_catch_error(p)
+    
+@pytest.mark.asyncio
+async def test_threaded_error_handling():
+    p = task(f3, workers=2)
+    assert await _try_catch_error(p)
+    
+@pytest.mark.asyncio
+async def test_multiprocessed_error_handling():
+    p = task(f3, workers=2, multiprocess=True)
+    assert await _try_catch_error(p)
+    
+@pytest.mark.asyncio
+async def test_unified_pipeline():
+    p = task(af1) | task(f1) | task(f2, branch=True, multiprocess=True) > consumer
+    assert await p(1) == 1
